package java.nio;

import sun.misc.Cleaner;
import sun.misc.Unsafe;
import sun.nio.ch.DirectBuffer;

class DirectLongBufferS extends LongBuffer implements DirectBuffer
{
    protected static final Unsafe unsafe;
    private static final long arrayBaseOffset;
    protected static final boolean unaligned;
    private final Object att;
    
    @Override
    public Object attachment() {
        return this.att;
    }
    
    public Cleaner cleaner() {
        return null;
    }
    
    DirectLongBufferS(DirectBuffer att, int n, int n2, int n3, int n4, int n5) {
        super(n, n2, n3, n4);
        this.address = att.address() + (long)n5;
        this.att = att;
    }
    
    @Override
    public LongBuffer slice() {
        int position = this.position();
        int limit = this.limit();
        int n = (position <= limit) ? (limit - position) : 0;
        int n2 = position << 3;
        assert n2 >= 0;
        return new DirectLongBufferS(this, -1, 0, n, n, n2);
    }
    
    @Override
    public LongBuffer duplicate() {
        return new DirectLongBufferS(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public LongBuffer asReadOnlyBuffer() {
        return new DirectLongBufferRS(this, this.markValue(), this.position(), this.limit(), this.capacity(), 0);
    }
    
    @Override
    public long address() {
        return this.address;
    }
    
    private long ix(int n) {
        return this.address + ((long)n << 3);
    }
    
    @Override
    public long get() {
        return Bits.swap(DirectLongBufferS.unsafe.getLong(this.ix(this.nextGetIndex())));
    }
    
    @Override
    public long get(int n) {
        return Bits.swap(DirectLongBufferS.unsafe.getLong(this.ix(this.checkIndex(n))));
    }
    
    @Override
    public LongBuffer get(long[] array, int n, int n2) {
        if ((long)n2 << 3 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferUnderflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyToLongArray(this.ix(position), array, (long)n << 3, (long)n2 << 3);
            }
            else {
                Bits.copyToArray(this.ix(position), array, DirectLongBufferS.arrayBaseOffset, (long)n << 3, (long)n2 << 3);
            }
            this.position(position + n2);
        }
        else {
            super.get(array, n, n2);
        }
        return this;
    }
    
    @Override
    public LongBuffer put(long n) {
        DirectLongBufferS.unsafe.putLong(this.ix(this.nextPutIndex()), Bits.swap(n));
        return this;
    }
    
    @Override
    public LongBuffer put(int n, long n2) {
        DirectLongBufferS.unsafe.putLong(this.ix(this.checkIndex(n)), Bits.swap(n2));
        return this;
    }
    
    @Override
    public LongBuffer put(LongBuffer longBuffer) {
        if ((longBuffer instanceof DirectLongBufferS)) {
            if (longBuffer == this) {
                throw new IllegalArgumentException();
            }
            DirectLongBufferS directLongBufferS = (DirectLongBufferS)longBuffer;
            int position2 = directLongBufferS.position();
            int limit2 = directLongBufferS.limit();
            assert position2 <= limit2;
            int n2 = (position2 <= limit2) ? (limit2 - position2) : 0;
            int position3 = this.position();
            int limit3 = this.limit();
            assert position3 <= limit3;
            if (n2 > ((position3 <= limit3) ? (limit3 - position3) : 0)) {
                throw new BufferOverflowException();
            }
            DirectLongBufferS.unsafe.copyMemory(directLongBufferS.ix(position2), this.ix(position3), (long)n2 << 3);
            directLongBufferS.position(position2 + n2);
            this.position(position3 + n2);
        }
        else if (longBuffer.hb != null) {
            int position = longBuffer.position();
            int limit = longBuffer.limit();
            assert position <= limit;
            int n = (position <= limit) ? (limit - position) : 0;
            this.put(longBuffer.hb, longBuffer.offset + position, n);
            longBuffer.position(position + n);
        }
        else {
            super.put(longBuffer);
        }
        return this;
    }
    
    @Override
    public LongBuffer put(long[] array, int n, int n2) {
        if ((long)n2 << 3 > 6L) {
            Buffer.checkBounds(n, n2, array.length);
            int position = this.position();
            int limit = this.limit();
            assert position <= limit;
            if (n2 > ((position <= limit) ? (limit - position) : 0)) {
                throw new BufferOverflowException();
            }
            if (this.order() != ByteOrder.nativeOrder()) {
                Bits.copyFromLongArray(array, (long)n << 3, this.ix(position), (long)n2 << 3);
            }
            else {
                Bits.copyFromArray(array, DirectLongBufferS.arrayBaseOffset, (long)n << 3, this.ix(position), (long)n2 << 3);
            }
            this.position(position + n2);
        }
        else {
            super.put(array, n, n2);
        }
        return this;
    }
    
    @Override
    public LongBuffer compact() {
        int position = this.position();
        int limit = this.limit();
        assert position <= limit;
        int n = (position <= limit) ? (limit - position) : 0;
        DirectLongBufferS.unsafe.copyMemory(this.ix(position), this.ix(0), (long)n << 3);
        this.position(n);
        this.limit(this.capacity());
        this.discardMark();
        return this;
    }
    
    @Override
    public boolean isDirect() {
        return true;
    }
    
    @Override
    public boolean isReadOnly() {
        return false;
    }
    
    @Override
    public ByteOrder order() {
        return (ByteOrder.nativeOrder() == ByteOrder.BIG_ENDIAN) ? ByteOrder.LITTLE_ENDIAN : ByteOrder.BIG_ENDIAN;
    }
    
    static {
        unsafe = Bits.unsafe();
        arrayBaseOffset = (long)DirectLongBufferS.unsafe.arrayBaseOffset(long[].class);
        unaligned = Bits.unaligned();
    }
}
